package com.qmth.themis.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.core.solar.config.SolarProperties;
import com.qmth.boot.core.solar.model.AppInfo;
import com.qmth.boot.core.solar.model.OrgInfo;
import com.qmth.boot.core.solar.service.SolarService;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dao.TBOrgMapper;
import com.qmth.themis.business.dao.TSAuthMapper;
import com.qmth.themis.business.dto.AuthOrgInfoDto;
import com.qmth.themis.business.entity.TBOrg;
import com.qmth.themis.business.entity.TSAuth;
import com.qmth.themis.business.enums.AuthEnum;
import com.qmth.themis.business.enums.UploadFileEnum;
import com.qmth.themis.business.service.AuthInfoService;
import com.qmth.themis.business.service.CacheService;
import com.qmth.themis.business.service.TBOrgService;
import com.qmth.themis.business.service.TSAuthService;
import com.qmth.themis.business.util.OssUtil;
import com.qmth.themis.business.util.QrCodeUtil;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.File;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @Description: 授权信息service
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2022/5/17
 */
@Service
public class AuthInfoServiceImpl implements AuthInfoService {
    private final static Logger log = LoggerFactory.getLogger(AuthInfoServiceImpl.class);

    @Resource
    SolarProperties solarProperties;

    @Resource
    SolarService solarService;

    @Resource
    TSAuthService tsAuthService;

    @Resource
    TSAuthMapper tsAuthMapper;

    @Resource
    CacheService cacheService;

    @Resource
    TBOrgService tbOrgService;

    @Resource
    TBOrgMapper tbOrgMapper;

    @Resource
    QrCodeUtil qrCodeUtil;

    @Resource
    AuthInfoService authInfoService;

    @Resource
    OssUtil ossUtil;

    /**
     * 授权信息初始化
     *
     * @return
     */
    @Override
    public AppInfo appInfoInit() {
        AppInfo appInfo = null;
        try {
            appInfo = solarService.getAppInfo();
            if (Objects.nonNull(appInfo) && Objects.nonNull(solarProperties)) {
                if (Objects.nonNull(solarProperties.getAccessKey())
                        && !Objects.equals(solarProperties.getAccessKey().trim(), "")
                        && Objects.nonNull(solarProperties.getAccessSecret())
                        && !Objects.equals(solarProperties.getAccessSecret().trim(), "")) {//在线激活
                    authInfoService.saveAuthInfo(appInfo, AuthEnum.ON_LINE, null);
                } else if (Objects.nonNull(solarProperties.getLicense())
                        && !Objects.equals(solarProperties.getLicense().trim(), "")) {//离线激活
                    authInfoService.saveAuthInfo(appInfo, AuthEnum.OFF_LINE, null);
                }
            } else {
                QueryWrapper<TSAuth> tsAuthQueryWrapper = new QueryWrapper<>();
                tsAuthQueryWrapper.lambda().isNotNull(TSAuth::getFile);
                List<TSAuth> tsAuthList = tsAuthService.list(tsAuthQueryWrapper);
                if (!CollectionUtils.isEmpty(tsAuthList)) {
                    for (TSAuth t : tsAuthList) {
                        appInfo = solarService.update(t.getFile());
                        authInfoService.saveAuthInfo(appInfo, AuthEnum.OFF_LINE, t.getFile());
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return appInfo;
    }

    /**
     * app是否过期
     *
     * @param code
     * @return
     */
    @Override
    public void appHasExpired(String code) {
        if (Objects.nonNull(code)) {
            AuthOrgInfoDto authOrgInfoDto = cacheService.authInfoCache(code);
            if (Objects.isNull(authOrgInfoDto) || (Objects.nonNull(authOrgInfoDto) && authOrgInfoDto.getControl().hasExpired())) {
                throw new BusinessException(ExceptionResultEnum.AUTH_INFO_ERROR);
            }
        }
    }

    /**
     * 离线激活
     *
     * @param licenseData
     */
    @Override
    public void updateLicense(byte[] licenseData) throws Exception {
        AppInfo appInfo = solarService.update(licenseData);
        if (Objects.isNull(appInfo)) {
            throw new BusinessException("激活失败");
        }
        authInfoService.saveAuthInfo(appInfo, AuthEnum.OFF_LINE, licenseData);
    }

    /**
     * 查询授权信息
     *
     * @return
     */
    @Override
    public Long selectAuthInfo() {
        Long expireTime = null;
        AppInfo appInfo = solarService.getAppInfo();
        if (Objects.nonNull(appInfo) && Objects.nonNull(appInfo.getControl())) {
            expireTime = Objects.nonNull(appInfo.getControl().getExpireTime()) ? appInfo.getControl().getExpireTime() : -1;
        }
        return expireTime;
    }

    /**
     * 保存鉴权信息
     *
     * @param appInfo
     * @param authEnum
     * @param file
     * @throws Exception
     */
    @Override
    @Transactional
    public void saveAuthInfo(AppInfo appInfo, AuthEnum authEnum, byte[] file) throws Exception {
        List<OrgInfo> orgInfoList = solarService.getOrgList();
        List<TSAuth> tsAuthList = null;
        Set<Long> orgIdsSet = null;
        Set<TBOrg> tbOrgSet = null;
        if (!CollectionUtils.isEmpty(orgInfoList)) {
            tsAuthList = new ArrayList<>();
            orgIdsSet = new HashSet<>();
            tbOrgSet = new HashSet<>();
        }
        boolean oss = qrCodeUtil.getSysDomain().isOss();
        for (OrgInfo o : orgInfoList) {
            orgIdsSet.add(o.getId());
            if (authEnum == AuthEnum.OFF_LINE) {
                if (Objects.isNull(file)) {
                    tsAuthList.add(new TSAuth(o.getId(), solarProperties.getLicense(), authEnum, appInfo.getControl().getExpireTime()));
                } else {
                    tsAuthList.add(new TSAuth(o.getId(), file, authEnum, appInfo.getControl().getExpireTime()));
                }
            } else {
                tsAuthList.add(new TSAuth(o.getId(), solarProperties.getAccessKey(), solarProperties.getAccessSecret(), authEnum, appInfo.getControl().getExpireTime()));
            }
            cacheService.updateAuthInfoCache(o.getCode());

            QueryWrapper<TBOrg> tbOrgQueryWrapper = new QueryWrapper<>();
            tbOrgQueryWrapper.lambda().eq(TBOrg::getCode, o.getCode());
            TBOrg tbOrg = tbOrgService.getOne(tbOrgQueryWrapper);
            if (Objects.isNull(tbOrg)) {//不存在则创建学校
                tbOrg = new TBOrg(o.getId(), o.getCode(), o.getName(), o.getAccessKey(), o.getAccessSecret());
                if (Objects.nonNull(o.getLogo()) && (!o.getLogo().startsWith("https:") || !o.getLogo().startsWith("http"))) {
                    String filePath = SystemConstant.TEMP_FILES_DIR + File.separator + SystemConstant.getUuid() + ".jpg";
                    File logoFile = new File(filePath);
                    if (!logoFile.getParentFile().exists()) {
                        // 不存在则创建父目录及子文件
                        logoFile.getParentFile().mkdirs();
                        logoFile.createNewFile();
                    }
                    SystemConstant.base64ToImage(o.getLogo(), filePath);

                    if (oss) {
                        LocalDateTime nowTime = LocalDateTime.now();
                        StringJoiner stringJoiner = new StringJoiner("");
                        stringJoiner.add(UploadFileEnum.file.name().toLowerCase()).add(File.separator)
                                .add(String.valueOf(nowTime.getYear())).add(File.separator)
                                .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                                .add(String.format("%02d", nowTime.getDayOfMonth())).add(File.separator)
                                .add(SystemConstant.getUuid()).add(".jpg");
                        ossUtil.upload(false, stringJoiner.toString(), logoFile);
                        logoFile.delete();
                        tbOrg.setLogo(ossUtil.getPrivateUrl(stringJoiner.toString()));
                    } else {
                        tbOrg.setLogo(filePath);
                    }
                } else {
                    tbOrg.setLogo(o.getLogo());
                }
                tbOrgSet.add(tbOrg);
            }
        }

        if (!CollectionUtils.isEmpty(tsAuthList) && !CollectionUtils.isEmpty(orgIdsSet)) {
            QueryWrapper<TSAuth> tsAuthQueryWrapper = new QueryWrapper<>();
            tsAuthQueryWrapper.lambda().in(TSAuth::getSchoolId, orgIdsSet);
            tsAuthService.remove(tsAuthQueryWrapper);
            tsAuthMapper.insertBatch(tsAuthList);

            if (!CollectionUtils.isEmpty(tbOrgSet)) {
                cacheService.removeOrgIdCache();
                cacheService.removeOrgCodeCache();
                tbOrgMapper.insertBatch(tbOrgSet);
            }
        }
    }
}
